/* -LICENSE-START-
 ** Copyright (c) 2018 Blackmagic Design
 **
 ** Permission is hereby granted, free of charge, to any person or organization
 ** obtaining a copy of the software and accompanying documentation covered by
 ** this license (the "Software") to use, reproduce, display, distribute,
 ** execute, and transmit the Software, and to prepare derivative works of the
 ** Software, and to permit third-parties to whom the Software is furnished to
 ** do so, all subject to the following:
 **
 ** The copyright notices in the Software and this entire statement, including
 ** the above license grant, this restriction and the following disclaimer,
 ** must be included in all copies of the Software, in whole or in part, and
 ** all derivative works of the Software, unless such copies or derivative
 ** works are solely in the form of machine-executable object code generated by
 ** a source language processor.
 **
 ** THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 ** IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 ** FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 ** SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 ** FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 ** ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 ** DEALINGS IN THE SOFTWARE.
 ** -LICENSE-END-
 */

import Foundation

struct CardStatus {
    var cardStatus = CCUPacketTypes.MediaStatus.None
    var remainingRecordTime: String = String.Localized("Transport.NoCard")
    var recordTimeWarning: RecordTimeWarning = RecordTimeWarning.NoWarning
}

struct CameraState {
    var whiteBalance: Int16
    var tint: Int16
    var customWhiteBalance: Int16
    var customTint: Int16
    var autoWhiteBalancePacketsExpected: Int
    var fStopIndex: Int
    var shutterAngle: Double
    var ISO: Int
    var gain: Int
    var shutterSpeed: Int32
    var recordingFormatData: CCUPacketTypes.RecordingFormatData
    var frameRateForShutterCalculations: Int16
    var cardStatuses: [CardStatus]
    var remainingTimeStrings: [String]
    var mediaStatuses: [CCUPacketTypes.MediaStatus]
    var hasRecordingError: Bool
    var reel: Int
    var scene: String
    var sceneTag: CCUPacketTypes.MetadataSceneTag
    var locationTag: CCUPacketTypes.MetadataLocationTypeTag
    var timeTag: CCUPacketTypes.MetadataDayNightTag
    var goodTake: Bool
    var takeNumber: Int
    var takeTag: CCUPacketTypes.MetadataTakeTag
    var slateName: String
    var slateType: CCUPacketTypes.MetadataSlateForType
    var transportInfo: TransportInfo
    var autoExposureMode: CCUPacketTypes.AutoExposureMode
    var timecode: String

    var expectedWhiteBalance = ExpectedValues<Int16>(errorTolerance: 0)
    var expectedTint = ExpectedValues<Int16>(errorTolerance: 0)
    var expectedFStopIndex = ExpectedValues<Int>(errorTolerance: 0)
	var expectedShutterSpeed = ExpectedValues<Int32>(errorTolerance: 1)
	var expectedShutterAngle = ExpectedValues<Int32>(errorTolerance: 0)
    var expectedOffSpeedFrameRate = ExpectedValues<Int16>(errorTolerance: 0)

    init() {
        whiteBalance = 2500
        tint = 0
        customWhiteBalance = 2500
        customTint = 0
        autoWhiteBalancePacketsExpected = 0
        fStopIndex = 0
        shutterAngle = 180.0
        ISO = 200
		gain = -6
        shutterSpeed = 0
        recordingFormatData = CCUPacketTypes.RecordingFormatData()
        frameRateForShutterCalculations = 0
        cardStatuses = Array(repeating: CardStatus(), count: 2)
        remainingTimeStrings = [String.Localized("Transport.NoCard"), String.Localized("Transport.NoCard")]
        mediaStatuses = [CCUPacketTypes.MediaStatus.None, CCUPacketTypes.MediaStatus.None]
        hasRecordingError = false
        reel = 0
        scene = ""
        sceneTag = CCUPacketTypes.MetadataSceneTag.None
        locationTag = CCUPacketTypes.MetadataLocationTypeTag.Exterior
        timeTag = CCUPacketTypes.MetadataDayNightTag.Night
        goodTake = false
        takeNumber = 0
        takeTag = CCUPacketTypes.MetadataTakeTag.None
        slateName = ""
        slateType = CCUPacketTypes.MetadataSlateForType.NextClip
        transportInfo = TransportInfo()
		autoExposureMode = CCUPacketTypes.AutoExposureMode.Manual
        timecode = "00:00:00:00"
    }
}
