/* -LICENSE-START-
 ** Copyright (c) 2018 Blackmagic Design
 **
 ** Permission is hereby granted, free of charge, to any person or organization
 ** obtaining a copy of the software and accompanying documentation covered by
 ** this license (the "Software") to use, reproduce, display, distribute,
 ** execute, and transmit the Software, and to prepare derivative works of the
 ** Software, and to permit third-parties to whom the Software is furnished to
 ** do so, all subject to the following:
 **
 ** The copyright notices in the Software and this entire statement, including
 ** the above license grant, this restriction and the following disclaimer,
 ** must be included in all copies of the Software, in whole or in part, and
 ** all derivative works of the Software, unless such copies or derivative
 ** works are solely in the form of machine-executable object code generated by
 ** a source language processor.
 **
 ** THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 ** IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 ** FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 ** SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 ** FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 ** ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 ** DEALINGS IN THE SOFTWARE.
 ** -LICENSE-END-
 */

import CoreBluetooth

// Bluetooth connection
public protocol InitialConnectionToUIDelegate: class {
    func updateDiscoveredPeripheralList(_ discoveredPeripheralList: [DiscoveredPeripheral])
    func transitionToCameraControl()
    func onSuccessfulPairing(_ peripheralName: String)
	func onPairingFailed(_ peripheralName: String, resolution: PairingFailureType)
    func onIncompatibleProtocolVersion(_ peripheralName: String, cameraVersion: Int, appVersion: Int)
}

public protocol InitialConnectionFromUIDelegate: class {
    func attemptConnection(to identifier: UUID)
    func disconnect()
    func refreshScan()
}

public protocol ConnectionManagerDelegate: class {
    func updateDiscoveredPeripheralList(_ discoveredPeripheralList: [DiscoveredPeripheral])
    func connectedToPeripheral(_ peripheral: CBPeripheral)
    func onConnectionLost()
    func onDisconnected(with error: Error?)
    func onReconnection()
    func isConnected() -> Bool
}

public protocol ConnectionStatusToUIDelegate: class {
    func onConnectionLost()
    func onDisconnection()
    func onReconnection()
    func onCameraPoweredOff()
    func onCameraPoweredOn()
    func onCameraReady()
}

// Incoming/Outgoing Packets
public protocol PacketReceivedDelegate: class {
    func onSuccessfulPairing(_ cameraName: String)
    func onTimecodePacketReceived(_ data: Data?)
    func onCameraStatusPacketReceived(_ data: Data?)
    func onCCUPacketReceived(_ data: Data?)
    func onCameraModelPacketReceived(_ data: Data?)
    func onWriteValueErrorReported(_ error: Error)
    func onReadValueErrorReported(_ error: Error)
    func onCameraNameChanged(_ cameraName: String)
    func onProtocolVersionReceived(_ peripheral: CBPeripheral, _ data: Data?)
}

public protocol PacketDecodedDelegate: class {
    func onWhiteBalanceReceived(_ whiteBalance: Int16, _ tint: Int16)
    func onRecordingFormatReceived(_ recordingFormatData: CCUPacketTypes.RecordingFormatData)
    func onIrisReceived(_ fStopIndex: Int)
	func onExposureReceived(_ exposure: Int32)
    func onShutterSpeedReceived(_ shutterSpeed: Int32)
	func onShutterAngleReceived(_ shutterAngleX100: Int32)
    func onISOReceived(_ iso: Int) -> Void
	func onGainReceived(_ sensorGain: Int) -> Void
    func onTransportInfoReceived(_ transportInfo: TransportInfo)
	func onAutoExposureModeReceived(_ autoExposureMode: CCUPacketTypes.AutoExposureMode)
    func onTimecodeReceived(_ timecode: String)
    func onMediaStatusReceived(_ mediaStatuses: [CCUPacketTypes.MediaStatus])
    func onRecordTimeRemainingReceived(_ remainingRecordTimes: [String], _ remainingRecordTimesInMinutes: [Int16])

    func onReelReceived(_ reelNumber: Int16)
    func onSceneTagsReceived(_ sceneTag: CCUPacketTypes.MetadataSceneTag, _ locationTag: CCUPacketTypes.MetadataLocationTypeTag, _ dayOrNight: CCUPacketTypes.MetadataDayNightTag)
    func onSceneReceived(_ scene: String)
    func onTakeReceived(_ takeNumber: Int8, _ takeTag: CCUPacketTypes.MetadataTakeTag)
    func onGoodTakeReceived(_ goodTake: Bool)
    func onSlateForNameReceived(_ slateForName: String)
    func onSlateForTypeReceived(_ slateForType: CCUPacketTypes.MetadataSlateForType)
}

protocol PacketEncodedDelegate: class {
    func onCCUPacketEncoded(_ data: Data)
    func onPowerPacketEncoded(_ data: Data)
}

// Location Services
protocol LocationServicesDelegate: class {
    func onLocationReceived(_ latitide: UInt64, _ longitude: UInt64)
    func onLocationFailed(_ error: Error)
}

// Slate control
public protocol IncomingSlateToUIDelegate: class {
    func onReelReceived(_ reelNumber: Int16)
    func onSceneTagsReceived(_ sceneTagIndex: Int, _ locationTagIndex: Int, _ timeTagIndex: Int)
    func onSceneReceived(_ scene: String)
    func onTakeReceived(_ takeNumber: Int8, _ takeTagIndex: Int)
    func onGoodTakeReceived(_ goodTake: Bool)
    func onSlateForNameReceived(_ slateForName: String)
    func setSlateForNextClip(_ isSlateForNextClip: Bool)
}

public protocol OutgoingSlateFromUIDelegate: class {
    func onReelIncremented()
    func onReelDecremented()
    func onSceneTagSelected(sceneTagIndex: Int)
    func onLocationTagSelected(locationTagIndex: Int)
    func onTimeTagSelected(timeTagIndex: Int)
    func onSceneIncremented()
    func onSceneDecremented()
    func onTakeIncremented()
    func onTakeDecremented()
    func onTakeTagSelected(takeTagIndex: Int)
    func onGoodTakeToggled(_ goodTake: Bool)
}

public protocol OutgoingPowerFromUIDelegate: class {
    func onPowerOff()
}

// Camera control
public protocol IncomingCameraControlToUIDelegate: class {
    func onWhiteBalanceReceived(_ whiteBalance: Int16, _ tint: Int16, _ presetIndex: Int)
    func onOffSpeedFrameRateReceived(_ offSpeedEnabled: Bool, _ frameRate: Int16)
    func onIrisReceived(_ fStopIndex: Int)
    func onShutterAngleReceived(_ shutterAngle: Double)
	func onShutterSpeedReceived(_ shutterSpeed: Int32)
    func onISOReceived(_ isoIndex: Int)
	func onGainReceived(_ gainIndex: Int)
    func onTransportModeReceived(_ transportMode: CCUPacketTypes.MediaTransportMode)
	func setIrisControlEnabled(_ enabled: Bool)
	func setShutterControlEnabled(_ enabled: Bool)
}

public protocol OutgoingCameraControlFromUIDelegate: class {
    func onTintIncremented() -> (tint: Int16, presetIndex: Int)
    func onTintDecremented() -> (tint: Int16, presetIndex: Int)
    func onWhiteBalanceIncremented() -> (whiteBalance: Int16, presetIndex: Int)
    func onWhiteBalanceDecremented() -> (whiteBalance: Int16, presetIndex: Int)

    func onWhiteBalancePresetPressed(_ presetIndex: Int, _ currentlySelected: Bool, _ saveCustomValues: Bool) -> (whiteBalance: Int16, tint: Int16)?
    func onWhiteBalanceChanged(_ whiteBalance: Int16) -> Int
    func onTintChanged(_ tint: Int16) -> Int
    func onAutoWhiteBalancePressed()
    func onOffSpeedFrameRateToggled(_ offSpeedEnabled: Bool)
    func onOffSpeedFrameRateIncremented() -> Int16
    func onOffSpeedFrameRateDecremented() -> Int16
    func onOffSpeedFrameRateChanged(_ frameRate: Int16)

    func onIrisIncremented() -> Int
    func onIrisDecremented() -> Int
    func onIrisChanged(_ fStopIndex: Int)

    func onShutterIncremented() -> Double
    func onShutterDecremented() -> Double
	func onShutterChanged(_ shutter: Double)

    func onISOChanged(_ isoIndex: Int)
}

// Recording control
public protocol IncomingRecordControlToUIDelegate: class {
    func setRecording(_ isRecording: Bool)
    func onActiveDisksReceived(_ disk1Active: Bool, _ disk2Active: Bool)
    func onTimecodeReceived(_ timecode: String)
    func onRecordTimeRemainingReceived(_ remainingRecordTimes: [String])
    func onTimelapseReceived(_ isTimelapseActive: Bool)
    func setRecordingError(_ hasRecordingError: Bool)
    func setRecordTimeWarning(_ recordTimeWarning: RecordTimeWarning, _ cardIndex: Int)
    func showPlaybackView(for clipName: String, at playbackSpeed: Int)
    func hidePlaybackView()
    func updateClipName(_ clipName: String)
}

public protocol OutgoingRecordControlFromUIDelegate: class {
    func onRecordPressed()
    func onDisk1Pressed()
    func onDisk2Pressed()
}

public protocol CameraNameDelegate: class {
    func onCameraNameChanged(_ cameraName: String)
}

// Default implementations of the 'ToUI' protocols, so the client doesn't need to
// implement all methods of a protocol.

public extension InitialConnectionToUIDelegate {
	func updateDiscoveredPeripheralList(_ discoveredPeripheralList: [DiscoveredPeripheral]) {}
	func transitionToCameraControl() {}
	func onSuccessfulPairing(_ peripheralName: String) {}
	func onPairingFailed(_ peripheralName: String, resolution: PairingFailureType) {}
	func onIncompatibleProtocolVersion(_ peripheralName: String, cameraVersion: Int, appVersion: Int) {}
}

public extension ConnectionStatusToUIDelegate {
	func onConnectionLost() {}
	func onDisconnection() {}
	func onReconnection() {}
	func onCameraPoweredOff() {}
	func onCameraPoweredOn() {}
	func onCameraReady() {}
}

public extension PacketReceivedDelegate {
	func onSuccessfulPairing(_ cameraName: String) {}
	func onTimecodePacketReceived(_ data: Data?) {}
	func onCameraStatusPacketReceived(_ data: Data?) {}
	func onCCUPacketReceived(_ data: Data?) {}
	func onCameraModelPacketReceived(_ data: Data?) {}
	func onWriteValueErrorReported(_ error: Error) {}
	func onReadValueErrorReported(_ error: Error) {}
	func onCameraNameChanged(_ cameraName: String) {}
	func onProtocolVersionReceived(_ peripheral: CBPeripheral, _ data: Data?) {}
}

public extension IncomingSlateToUIDelegate {
	func onReelReceived(_ reelNumber: Int16) {}
	func onSceneTagsReceived(_ sceneTagIndex: Int, _ locationTagIndex: Int, _ timeTagIndex: Int) {}
	func onSceneReceived(_ scene: String) {}
	func onTakeReceived(_ takeNumber: Int8, _ takeTagIndex: Int) {}
	func onGoodTakeReceived(_ goodTake: Bool) {}
	func onSlateForNameReceived(_ slateForName: String) {}
	func setSlateForNextClip(_ isSlateForNextClip: Bool) {}
}

public extension IncomingCameraControlToUIDelegate {
	func onWhiteBalanceReceived(_ whiteBalance: Int16, _ tint: Int16, _ presetIndex: Int) {}
	func onOffSpeedFrameRateReceived(_ offSpeedEnabled: Bool, _ frameRate: Int16) {}
	func onIrisReceived(_ fStopIndex: Int) {}
	func onShutterAngleReceived(_ shutterAngle: Double) {}
	func onShutterSpeedReceived(_ shutterSpeed: Int32) {}
	func onISOReceived(_ isoIndex: Int) {}
	func onGainReceived(_ gainIndex: Int) {}
	func onTransportModeReceived(_ transportMode: CCUPacketTypes.MediaTransportMode) {}
	func setIrisControlEnabled(_ enabled: Bool) {}
	func setShutterControlEnabled(_ enabled: Bool) {}
}

public extension IncomingRecordControlToUIDelegate {
	func setRecording(_ isRecording: Bool) {}
	func onActiveDisksReceived(_ disk1Active: Bool, _ disk2Active: Bool) {}
	func onTimecodeReceived(_ timecode: String) {}
	func onRecordTimeRemainingReceived(_ remainingRecordTimes: [String]) {}
	func onTimelapseReceived(_ isTimelapseActive: Bool) {}
	func setRecordingError(_ hasRecordingError: Bool) {}
	func setRecordTimeWarning(_ recordTimeWarning: RecordTimeWarning, _ cardIndex: Int) {}
	func showPlaybackView(for clipName: String, at playbackSpeed: Int) {}
	func hidePlaybackView() {}
	func updateClipName(_ clipName: String) {}
}
