/* -LICENSE-START-
 ** Copyright (c) 2018 Blackmagic Design
 **
 ** Permission is hereby granted, free of charge, to any person or organization
 ** obtaining a copy of the software and accompanying documentation covered by
 ** this license (the "Software") to use, reproduce, display, distribute,
 ** execute, and transmit the Software, and to prepare derivative works of the
 ** Software, and to permit third-parties to whom the Software is furnished to
 ** do so, all subject to the following:
 **
 ** The copyright notices in the Software and this entire statement, including
 ** the above license grant, this restriction and the following disclaimer,
 ** must be included in all copies of the Software, in whole or in part, and
 ** all derivative works of the Software, unless such copies or derivative
 ** works are solely in the form of machine-executable object code generated by
 ** a source language processor.
 **
 ** THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 ** IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 ** FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 ** SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 ** FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 ** ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 ** DEALINGS IN THE SOFTWARE.
 ** -LICENSE-END-
 */

import Foundation

public struct LensConfig {
    public static let fStopValues: [Double] = [1.2, 1.4, 1.8, 2.0, 2.2, 2.4, 2.6, 2.8, 3.2, 3.5, 3.7, 4.0, 4.5, 4.8, 5.2, 5.6, 6.2, 6.7, 7.3, 8.0, 8.7, 9.5, 10.0, 11.0, 12.0, 14.0, 15.0, 16.0, 17.0, 19.0, 21.0, 22.0]

    // ccu_fixed_t representation of log2(fstop ^ 2) (see definition and conversion functions in CCUPacketTypes.swift)
    public static let apertureNumbers: [Int16] = [1077, 1988, 3473, 4096, 4659, 5173, 5646, 6084, 6873, 7402, 7731, 8192, 8888, 9269, 9742, 10180, 10781, 11240, 11746, 12288, 12783, 13303, 13606, 14169, 14684, 15594, 16002, 16384, 16742, 17399, 17990, 18265]

    public static func GetIndexForApertureNumber(_ targetNumber: Int16) -> Int {
        if targetNumber > apertureNumbers.last! {
            return apertureNumbers.count - 1
        }

        if let index: Int = apertureNumbers.index(where: { targetNumber <= $0 }) {
            return index
        }

        return -1
    }

    public static func GetFStopString(_ fStopValue: Double) -> String {
        let isFractionalNumber: Bool = fStopValue != floor(fStopValue)
        let isLessThanTen: Bool = fStopValue < 10.0
        if isFractionalNumber || isLessThanTen {
            return String(format: "f%.1f", fStopValue)
        }

        return String(format: "f%d", Int(fStopValue))
    }
}
